@@ -28,6 +28,7 @@ use std::thread::JoinHandle;
 use std::{io, thread};
 
 use crate::common::{self, MESSAGE_TYPE_REQUEST};
+use crate::context;
 use crate::error::{get_status, Error, Result};
 use crate::sync::channel::{read_message, write_message};
 use crate::ttrpc::{Code, Request, Response};
@@ -209,6 +210,7 @@ fn start_method_handler_thread(
                 fd,
                 mh,
                 res_tx: res_tx.clone(),
+                metadata: context::from_pb(&req.metadata),
             };
             if let Err(x) = method.handler(ctx, req) {
                 debug!("method handle {} get error {:?}", path, x);
@@ -247,7 +249,7 @@ fn start_method_handler_threads(num: usize, ts: &ThreadS) {
 fn check_method_handler_threads(ts: &ThreadS) {
     let c = ts.wtc.load(Ordering::SeqCst);
     if c < ts.min {
-        start_method_handler_threads(ts.default - c, ts);
+        start_method_handler_threads(ts.default - c, &ts);
     }
 }
 
@@ -350,10 +352,7 @@ impl Server {
                                 .unwrap()
                                 .remove(&fd)
                                 .map(|mut cn| {
-                                    cn.handler.take().map(|handler| {
-                                        handler.join().unwrap();
-                                        close(fd).unwrap();
-                                    })
+                                    cn.handler.take().map(|handler| handler.join().unwrap())
                                 });
                         }
                         info!("reaper thread exited");
@@ -481,8 +480,7 @@ impl Server {
                             // drop the res_tx, thus the res_rx would get terminated notification.
                             drop(res_tx);
                             handler.join().unwrap_or(());
-                            // client_handler should not close fd before exit
-                            // , which prevent fd reuse issue.
+                            close(fd).unwrap_or(());
                             reaper_tx_child.send(fd).unwrap();
 
                             debug!("client thread quit");
